/////////////////////////////////////////////////////////////////
//
// pbo.cpp
// (c)2003 by Ales Mlakar aka JaZzBrE (jazzbre@xubl.com)
//
/////////////////////////////////////////////////////////////////

#include "pbo.h"
#include <stdio.h>
#include <direct.h>
#include <io.h>

/////////////////////////////////////////////////////////////////
// 
// PboEntry
//
/////////////////////////////////////////////////////////////////

PboEntry::PboEntry()
{
   filename[0]=0;
   size=0;
   memset(unknown,0,16);
}

int      PboEntry::Read(FILE *f)
{   
   while(1)
   {
      char c=fgetc(f);

      if (c==char(0))
         break;         

      filename+=c;
   }

   fread(unknown,1,16,f);
   fread(&size,1,4,f);

   if (size==0)
      return 0;

   return 1;
}


int      PboEntry::Write(FILE *f)
{
   fwrite(filename.c_str(),1,filename.size(),f);
   fputc(char(0),f);
   fwrite(unknown,1,16,f);
   fwrite(&size,1,4,f);

   return 1;
}


/////////////////////////////////////////////////////////////////
//
// PboPack
//
/////////////////////////////////////////////////////////////////

PboPack::PboPack()
{
   pPBOFile=0;
   curfile=0;
   curprocessfile=0;
   m_ProcessDataProc=0;
}

PboPack::~PboPack()
{  
   for (PboEntryVector::iterator i=Entry.begin();i<Entry.end();i++)
   {
      delete (*i);
   }
}

int   PboPack::Open(const char *name)
{
   CurName=name;

   pPBOFile=fopen(name,"rb");
   if (!pPBOFile)
      return 0;

   if (!ReadHeader())
      return 0;

   
   return 1;
}

int   PboPack::Close()
{
   fclose(pPBOFile);
   return 1;
}

int   PboPack::ReadHeader()
{
   while(1)
   {
      PboEntry *p=new PboEntry;

      int ret = p->Read(pPBOFile);

      Entry.push_back(p);
      
      if (!ret)
         break;
   }

   int pos = ftell( pPBOFile );
   int readstart = pos;

   for (int i=0;i<Entry.size();i++)
   {
      PboEntry *e = Entry[i];
      e->unknown[0] = pos; 
      pos+=e->size;
   }

   fseek( pPBOFile, readstart, 0 );

   return 1;
}

int   PboPack::ExtractFile(PboEntry *ent, const char *outname)
{
   fseek( pPBOFile, ent->unknown[0], 0 );

   m_ProcessDataProc( (char *)ent->filename.c_str(), ent->size ); 

   char  *buf=new char[ent->size];   
   fread( buf, 1, ent->size, pPBOFile );

   FILE  *pout=fopen(outname, "wb");
   fwrite(buf, 1, ent->size, pout);
   fclose(pout);

   delete[]buf;

   return 0;   
}

int   PboPack::CompressFiles(const vector<string> &fnames, const char *path, const char *outname)
{
   /////////////////////////////////////////////////////////////////
   //
   /////////////////////////////////////////////////////////////////

   for (int i=0;i<fnames.size();i++)
   {
      PboEntry *ent=new PboEntry;
      ent->filename = fnames[i];      

      string   pfilename = string(path) + ent->filename;

      FILE  *f=fopen(  pfilename.c_str() , "rb" );
      if (f)
      {
         ent->size = filelength(fileno(f));
         fclose(f);
      }
      else
      {
         delete( ent );
         continue;
      }

      Entry.push_back( ent );
   }

   PboEntry *ent=new PboEntry;
   Entry.push_back( ent );

   /////////////////////////////////////////////////////////////////
   //
   /////////////////////////////////////////////////////////////////


   FILE  *of=fopen(outname,"wb");

   for (PboEntryVector::iterator i=Entry.begin();i<Entry.end();i++)
   {
      PboEntry *p=(*i);

      p->Write(of);
   }

   for (i=Entry.begin();i<Entry.end();i++)
   {
      PboEntry *ent=(*i);
      //
      string   pfilename = string(path) + ent->filename;
      //
      FILE  *f=fopen(  pfilename.c_str() , "rb" );
      if (f)
      {
         m_ProcessDataProc( (char *)ent->filename.c_str(), ent->size ); 

         char *buf=new char[ent->size];
         fread(buf, 1, ent->size, f );
         fwrite(buf, 1, ent->size, of );
         delete[]buf;
         fclose(f);
      }
   }

   const char  *cr=" ** PboTool(TotCmd) (c)2003 JaZzBrE ** ";

   fwrite(cr,1,strlen(cr),of);

   fclose(of);

   return 1;
}